<?php
/*+**********************************************************************************
 *	Skebby Implementation on vtiger 6 plugin
 *
 * Type: classic OR classic_plus OR basic
 * Prefix: 39 for italy, is valid only for recipient, not sender
 * From: alphanumeric string (max 11 chars) or numeric sender
 * 
 * IMPORTANT
 * 
 * In order to make SMSNotifier Module works, you have to change line 35, in /modules/SMSNotifier/models/Provider.php
 *
 * Change this:
 *  
 *	    if (!in_array($file, array('.', '..', 'MyProvider.php', 'CVS'))) {
 * 
 * to this:
 *
 *		if (!in_array($file, array('.', '..', 'MyProvider.php', 'SMSProvider.php'))) {
 * 
 ************************************************************************************/
 
define('NET_ERROR', 'Errore+di+rete+impossibile+spedire+il+messaggio');
define('SENDER_ERROR', 'Puoi+specificare+solo+un+tipo+di+mittente,+numerico+o+alfanumerico');

define ('SMS_TYPE_CLASSIC', 'classic');
define ('SMS_TYPE_CLASSIC_PLUS', 'classic_plus');
define ('SMS_TYPE_BASIC', 'basic');

class SMSNotifier_Skebby_Provider implements SMSNotifier_ISMSProvider_Model {

	private $userName;
	private $password;
	private $parameters = array();

	const SERVICE_URI = 'http://gateway.skebby.it'; 
	private static $REQUIRED_PARAMETERS = array('Type','From','Prefix'); // parameters specific of Skebby

	/**
	 * Function to get provider name
	 * @return <String> provider name
	 */
	public function getName() {
		return 'Skebby';
	}

	/**
	 * Function to get required parameters other than (userName, password)
	 * @return <array> required parameters list
	 */
	public function getRequiredParams() {
		return self::$REQUIRED_PARAMETERS;
	}

	/**
	 * Function to get service URL to use for a given type
	 * @param <String> $type like SEND, PING, QUERY
	 */
	public function getServiceURL($type = false) {
		if($type) {
			switch(strtoupper($type)) {
				case self::SERVICE_AUTH: return  self::SERVICE_URI . '/http/auth';
				case self::SERVICE_SEND: return  self::SERVICE_URI . '/api/send/smseasy/advanced/http.php';
				case self::SERVICE_QUERY: return self::SERVICE_URI . '/http/querymsg';
			}
		}
		return false;
	}

	/**
	 * Function to set authentication parameters
	 * @param <String> $userName
	 * @param <String> $password
	 */
	public function setAuthParameters($userName, $password) {
		$this->userName = $userName;
		$this->password = $password;
	}

	/**
	 * Function to set non-auth parameter.
	 * @param <String> $key
	 * @param <String> $value
	 */
	public function setParameter($key, $value) {
		$this->parameters[$key] = $value;
	}

	/**
	 * Function to get parameter value
	 * @param <String> $key
	 * @param <String> $defaultValue
	 * @return <String> value/$default value
	 */
	public function getParameter($key, $defaultValue = false) {
		if(isset($this->parameters[$key])) {
			return $this->parameters[$key];
		}
		return $defaultValue;
	}

	/**
	 * Function to prepare parameters
	 * @return <Array> parameters
	 */
	protected function prepareParameters() {
		$params = array('username' => $this->userName, 'password' => $this->password);
		foreach (self::$REQUIRED_PARAMETERS as $key) {
			$params[$key] = $this->getParameter($key);
		}
		return $params;
	}

	/**
	 * Function to handle SMS Send operation
	 * @param <String> $message
	 * @param <Mixed> $recipients One or Array of numbers
	 */
	public function send($message, $recipients) {
		if(!is_array($recipients)) {
			$recipients = array($recipients);
		}

		$params = $this->prepareParameters();

			foreach($recipients as $key => $value){
				if($params['Prefix'] && is_numeric($params['Prefix'])){
					$finalRecipient = $params['Prefix'].$value;				
				}
				else{
					$finalRecipient = $value;				
				}

				// strip all non numeric 
				$finalRecipient = preg_replace('/[^0-9]+/', '', $finalRecipient);

				// strip leading 0 and +
				$finalRecipient = ltrim($finalRecipient, '0+');
				$recipients[$key] = $finalRecipient;
			}

		$smsType = $params['Type'] ? strtolower($params['Type']) : 'classic_plus';

		switch($smsType){
			case 'basic':
				$response = $this->skebbyGatewaySendSMS($params['username'],$params['password'],$recipients, $message, SMS_TYPE_BASIC);
			break;
			case 'classic':
			case 'classic_plus':
			default:
				$typeToSend = 'classic' == $smsType ? SMS_TYPE_CLASSIC : SMS_TYPE_CLASSIC_PLUS; 
				$sender = $params['From'] ? $params['From'] : 'SMS';

				if (is_numeric($sender)){
					// Invio SMS con mittente personalizzato di tipo numerico
					$response = $this->skebbyGatewaySendSMS($params['username'],$params['password'],$recipients,$message, $typeToSend,$sender);
				}
				else{
					// Invio SMS con mittente personalizzato di tipo alfanumerico
					$response = $this->skebbyGatewaySendSMS($params['username'],$params['password'],$recipients,$message, $typeToSend,'',$sender);
				}
			break;
		}

		$this->log('response: ' . print_r($response, true));

		$results = array();
		foreach($recipients as $to) {
			$result = array(  'to' => $to );
			if('success' == $response['status']){
				$result['id'] = $response['id'] ? $response['id'] : $to;
				$result['status'] = self::MSG_STATUS_DISPATCHED;
				$result['error'] = false;
				$result['statusmessage'] = 'Sent';
			}
			else{
				$result['status'] = self::MSG_STATUS_FAILED;
				$result['error'] = true;
				$result['statusmessage'] = $result['message'];	
			}
			$results[] = $result;
		}

		return $results;
	}

	/**
	 * Function to get query for status using messgae id
	 * @param <Number> $messageId
	 */
	public function query($messageid) {
		$result = array( 'error' => false, 'needlookup' => 1 );
		$result['status'] = self::MSG_STATUS_DISPATCHED;
		$result['needlookup'] = 0;
		return $result;
	}
	
	function do_post_request($url, $data, $optional_headers = null){
		if(!function_exists('curl_init')) {
			$params = array(
					'http' => array(
							'method' => 'POST',
							'content' => $data
					)
			);
			if ($optional_headers !== null) {
				$params['http']['header'] = $optional_headers;
			}
			$ctx = stream_context_create($params);
			$fp = @fopen($url, 'rb', false, $ctx);
			if (!$fp) {
				return 'status=failed&message='.NET_ERROR;
			}
			$response = @stream_get_contents($fp);
			if ($response === false) {
				return 'status=failed&message='.NET_ERROR;
			}
			return $response;
		} else {
			$ch = curl_init();
			curl_setopt($ch,CURLOPT_CONNECTTIMEOUT,10);
			curl_setopt($ch,CURLOPT_RETURNTRANSFER,true);
			curl_setopt($ch,CURLOPT_TIMEOUT,60);
			curl_setopt($ch,CURLOPT_USERAGENT,'Generic Client');
			curl_setopt($ch,CURLOPT_POSTFIELDS,$data);
			curl_setopt($ch,CURLOPT_URL,$url);
	
			if ($optional_headers !== null) {
				curl_setopt($ch,CURLOPT_HTTPHEADER,$optional_headers);
			}

			$response = curl_exec($ch);
			curl_close($ch);
			if(!$response){
				return 'status=failed&message='.NET_ERROR;
			}
			return $response;
		}
	}
	
	function skebbyGatewaySendSMS($username,$password,$recipients,$text,$sms_type=SMS_TYPE_CLASSIC,$sender_number='',$sender_string='',$user_reference='',$charset='',$optional_headers=null) {
		$url = $this->getServiceUrl(self::SERVICE_SEND);
	
		switch($sms_type) {
			case SMS_TYPE_CLASSIC:
			default:
				$method='send_sms_classic';
				break;
			case SMS_TYPE_CLASSIC_PLUS:
				$method='send_sms_classic_report';
				break;
			case SMS_TYPE_BASIC:
				$method='send_sms_basic';
				break;
		}
	
		$parameters = 'method='
			.urlencode($method).'&'
			.'username='.urlencode($username).'&'
			.'password='.urlencode($password).'&'
			.'text='.urlencode($text).'&'
			.'recipients[]='.implode('&recipients[]=',$recipients);
			
		if($sender_number != '' && $sender_string != '') {
			parse_str('status=failed&message='.SENDER_ERROR,$result);
			return $result;
		}
		$parameters .= $sender_number != '' ? '&sender_number='.urlencode($sender_number) : '';
		$parameters .= $sender_string != '' ? '&sender_string='.urlencode($sender_string) : '';
	
		$parameters .= $user_reference != '' ? '&user_reference='.urlencode($user_reference) : '';
	
	
		switch($charset) {
			case 'UTF-8':
				$parameters .= '&charset='.urlencode('UTF-8');
				break;
			case '':
			case 'ISO-8859-1':
			default:
				break;
		}
	
		$this->log('request'. $parameters);

		parse_str($this->do_post_request($url,$parameters,$optional_headers),$result);
	
		return $result;
	}

	function skebbyGatewayGetCredit($username,$password,$charset=''){
		$url = $this->getServiceUrl(self::SERVICE_SEND);
		$method = 'get_credit';
	
		$parameters = 'method='.urlencode($method).'&'
			.'username='.urlencode($username).'&'
			.'password='.urlencode($password);
	
		switch($charset) {
			case 'UTF-8':
				$parameters .= '&charset='.urlencode('UTF-8');
				break;
			default:
		}
	
		parse_str($this->do_post_request($url,$parameters),$result);
		return $result;
	}

	function log($text){
		if($this->enableLogging){			
			$fileName = '/tmp/skebby_logging.txt';
			$fp = fopen($fileName, 'a+');
			if($fp){
				flock($fp, LOCK_EX);
				fwrite($fp, $text . "\n");
				flock($fp, LOCK_UN);
				fclose($fp);
			}		
		}
	}
}
?>
